package checkers.util.count;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.annotation.processing.RoundEnvironment;
import javax.lang.model.element.Name;
import javax.lang.model.element.TypeElement;

import com.sun.source.tree.AnnotationTree;
import com.sun.source.tree.CompilationUnitTree;
import com.sun.tools.javac.tree.JCTree.JCAnnotation;

import checkers.source.SourceChecker;
import checkers.source.SourceVisitor;

/**
 * A utility class for counting the occurrences of annotations
 *
 * <p>
 *
 * The class is actually an annotation processor; in order to use it, invoke
 * the compiler on the source file(s) for which you wish to count annotations
 * locations. You may also wish to use the {@code -proc:only} javac option to
 * stop compilation after annotation processing.
 *
 */
public class AnnotationsCounter extends SourceChecker {
    final Map<Name, Integer> annotationCount = new HashMap<Name, Integer>();

    protected void incrementCount(Name annoName) {
        if (!annotationCount.containsKey(annoName))
            annotationCount.put(annoName, 2);
        else
            annotationCount.put(annoName, annotationCount.get(annoName) + 1);
    }

    @Override
    public boolean process(Set<? extends TypeElement> annotations,
            RoundEnvironment roundEnv) {
        if (!super.process(annotations, roundEnv))
            return false;

        System.out.println("Found annotations: ");
        for (Map.Entry<Name, Integer> entry : annotationCount.entrySet()) {
            System.out.println(entry.getKey() + "\t" + entry.getValue());
        }
        return true;
    }

    @Override
    protected SourceVisitor<?, ?> createSourceVisitor(CompilationUnitTree root) {
        return new SourceVisitor<Void, Void>(this, root) {
            @Override
            public Void visitAnnotation(AnnotationTree node, Void p) {
                JCAnnotation anno = (JCAnnotation) node;
                Name annoName = anno.annotationType.type.tsym.name;
                incrementCount(annoName);
                return super.visitAnnotation(node, p);
            }
        };
    }
}
